'use strict';

exports.__esModule = true;
exports.default = router;

var _effects = require('redux-saga/effects');

var _fsmIterator2 = require('fsm-iterator');

var _fsmIterator3 = _interopRequireDefault(_fsmIterator2);

var _buildRouteMatcher = require('./buildRouteMatcher');

var _buildRouteMatcher2 = _interopRequireDefault(_buildRouteMatcher);

var _createHistoryChannel = require('./createHistoryChannel');

var _createHistoryChannel2 = _interopRequireDefault(_createHistoryChannel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-console: ["error", { allow: ["error"] }] */
var INIT = 'INIT';
var LISTEN = 'LISTEN';
var HANDLE_LOCATION = 'HANDLE_LOCATION';

function router(history, routes) {
  var _fsmIterator;

  var routeMatcher = (0, _buildRouteMatcher2.default)(routes);
  var historyChannel = null;
  var lastMatch = null;

  function errorMessageValue(error, message) {
    var finalMessage = 'Redux Saga Router: ' + message + ':\n' + error.message;

    if ('stack' in error) {
      finalMessage += '\n' + error.stack;
    }

    return {
      value: (0, _effects.call)([console, console.error], finalMessage),
      next: LISTEN
    };
  }

  return (0, _fsmIterator3.default)(INIT, (_fsmIterator = {}, _fsmIterator[INIT] = function () {
    return {
      value: (0, _effects.call)(_createHistoryChannel2.default, history),
      next: LISTEN
    };
  }, _fsmIterator[LISTEN] = function (channel) {
    if (channel) {
      historyChannel = channel;
    }

    return {
      value: (0, _effects.take)(historyChannel),
      next: HANDLE_LOCATION
    };
  }, _fsmIterator[HANDLE_LOCATION] = function (location, fsm) {
    var path = location.pathname;
    var match = routeMatcher.match(path);

    if (match) {
      lastMatch = match;

      return {
        value: (0, _effects.call)(match.action, match.params),
        next: LISTEN
      };
    }

    return fsm[LISTEN]();
  }, _fsmIterator.throw = function _throw(e, fsm) {
    switch (fsm.previousState) {
      case HANDLE_LOCATION:
        return errorMessageValue(e, 'Unhandled ' + e.name + ' in route "' + lastMatch.route + '"');

      case LISTEN:
        return errorMessageValue(e, 'Unexpected ' + e.name + ' while listening for route');

      default:
        return { done: true };
    }
  }, _fsmIterator));
}